"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var history_1 = require("history");
var path_to_regexp_1 = tslib_1.__importDefault(require("path-to-regexp"));
var ForgetRegisterError_1 = require("@redux-model/web/core/exceptions/ForgetRegisterError");
var web_1 = require("@redux-model/web");
var history_2 = require("./history");
var RouterModel = /** @class */ (function (_super) {
    tslib_1.__extends(RouterModel, _super);
    function RouterModel() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.isRegistered = false;
        _this.timer = null;
        _this.pathListeners = [];
        _this.changeHistory = _this.actionNormal(function (_, payload) {
            return payload;
        });
        return _this;
    }
    RouterModel.prototype.push = function (path, state) {
        this.getHistory().push(path, state);
    };
    RouterModel.prototype.replace = function (path, state) {
        this.getHistory().replace(path, state);
    };
    RouterModel.prototype.go = function (index) {
        this.getHistory().go(index);
    };
    RouterModel.prototype.goBack = function () {
        this.getHistory().goBack();
    };
    RouterModel.prototype.goForward = function () {
        this.getHistory().goForward();
    };
    RouterModel.prototype.subscribe = function (path, fn) {
        var token = "un_" + this.pathListeners.length + "_" + Math.random();
        var keys = [];
        var reg = path_to_regexp_1.default(path, keys);
        var subscriber = { path: path, fn: fn, reg: reg, keys: keys, token: token };
        this.pathListeners.push(subscriber);
        if (this.isRegistered) {
            this.publishOne(subscriber, this.data.location, this.data.action);
        }
        return token;
    };
    RouterModel.prototype.unsubscribe = function (token) {
        this.pathListeners = this.pathListeners.filter(function (item) {
            return item.token !== token;
        });
    };
    RouterModel.prototype.registerBrowser = function (history) {
        var originalHistory = history_2.getHistory();
        var newHistory = history || originalHistory || history_1.createBrowserHistory();
        history_2.setHistory(newHistory);
        if (originalHistory && originalHistory !== newHistory && this.unregister) {
            this.unregister();
        }
        return this.register();
    };
    RouterModel.prototype.registerHash = function (history) {
        var originalHistory = history_2.getHistory();
        var newHistory = history || originalHistory || history_1.createHashHistory();
        history_2.setHistory(newHistory);
        if (originalHistory && originalHistory !== newHistory && this.unregister) {
            this.unregister();
        }
        return this.register();
    };
    RouterModel.prototype.register = function () {
        var _this = this;
        var history = history_2.getHistory();
        if (!history) {
            throw new ReferenceError('Use "registerBrowser()" or "registerHash()" for routerModel.');
        }
        if (!this.unregister) {
            this.unregister = history.listen(function (location, action) {
                _this.changeHistory({
                    location: location,
                    action: action,
                });
                _this.publishAll(location, action);
            });
        }
        return _super.prototype.register.call(this);
    };
    RouterModel.prototype.publishAll = function (location, action) {
        var _this = this;
        this.pathListeners.forEach(function (subscriber) {
            _this.publishOne(subscriber, location, action);
        });
    };
    RouterModel.prototype.publishOne = function (_a, location, action) {
        var fn = _a.fn, reg = _a.reg, keys = _a.keys;
        var result = reg.exec(location.pathname);
        if (result === null) {
            return;
        }
        var params = {};
        keys.forEach(function (_a, index) {
            var name = _a.name;
            params[name] = result[index + 1];
        });
        fn(params, location, action);
    };
    RouterModel.prototype.getHistory = function () {
        var history = history_2.getHistory();
        if (!history) {
            throw new ForgetRegisterError_1.ForgetRegisterError('RouterModel');
        }
        return history;
    };
    RouterModel.prototype.initReducer = function () {
        var _this = this;
        return function () {
            var history = _this.getHistory();
            // We must realize that redux will do assertion multiple times to make sure reducer response data has no problem.
            // Therefore this function will be invoked multiple times, we must clear previous timer.
            if (_this.timer) {
                clearTimeout(_this.timer);
            }
            _this.timer = setTimeout(function () {
                _this.publishAll(history.location, history.action);
            });
            _this.isRegistered = true;
            return {
                location: history.location,
                action: history.action,
            };
        };
    };
    return RouterModel;
}(web_1.Model));
exports.routerModel = new RouterModel();
